#! /usr/bin/env python2

from distutils.core import setup
setup(name="untwisted",
      version="1.0.0",
      packages=["untwisted", 
                "untwisted.plugins"],
      scripts=['quickserv'],
      author="Iury O. G. Figueiredo",
      author_email="ioliveira@id.uff.br",
      url='https://github.com/iogf/untwisted',
      download_url='https://github.com/iogf/untwisted/releases',
      keywords=['asynchronous', 'programming', 'twisted', 'untwisted', 'library', 'framework', 'networking', 'protocols', 'rapidserv', 'irc', 'requests'],
      classifiers=[],
      description="A library for asynchronous programming in python.",)
















